@@ -3,7 +3,18 @@
 import os
 import re
 import time
-from typing import Any, cast, Dict, List, Optional, Tuple, Type, TYPE_CHECKING, Union
+from typing import (
+    Any,
+    BinaryIO,
+    cast,
+    Dict,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TYPE_CHECKING,
+    Union,
+)
 from urllib import parse
 
 import requests
@@ -402,32 +413,6 @@ class Gitlab:
 
         return cast(str, self._server_version), cast(str, self._server_revision)
 
-    @gitlab.exceptions.on_http_error(gitlab.exceptions.GitlabVerifyError)
-    def lint(self, content: str, **kwargs: Any) -> Tuple[bool, List[str]]:
-        """Validate a gitlab CI configuration.
-
-        Args:
-            content: The .gitlab-ci.yml content
-            **kwargs: Extra options to send to the server (e.g. sudo)
-
-        Raises:
-            GitlabAuthenticationError: If authentication is not correct
-            GitlabVerifyError: If the validation could not be done
-
-        Returns:
-            (True, []) if the file is valid, (False, errors(list)) otherwise
-        """
-        utils.warn(
-            "`lint()` is deprecated and will be removed in a future version.\n"
-            "Please use `ci_lint.create()` instead.",
-            category=DeprecationWarning,
-        )
-        post_data = {"content": content}
-        data = self.http_post("/ci/lint", post_data=post_data, **kwargs)
-        if TYPE_CHECKING:
-            assert not isinstance(data, requests.Response)
-        return (data["status"] == "valid", data["errors"])
-
     @gitlab.exceptions.on_http_error(gitlab.exceptions.GitlabMarkdownError)
     def markdown(
         self, text: str, gfm: bool = False, project: Optional[str] = None, **kwargs: Any
@@ -511,49 +496,64 @@ class Gitlab:
             not self.http_username and self.http_password
         ):
             raise ValueError("Both http_username and http_password should be defined")
-        if self.oauth_token and self.http_username:
+        if tokens and self.http_username:
             raise ValueError(
-                "Only one of oauth authentication or http "
+                "Only one of token authentications or http "
                 "authentication should be defined"
             )
 
-        self._http_auth = None
+        self._auth: Optional[requests.auth.AuthBase] = None
         if self.private_token:
-            self.headers.pop("Authorization", None)
-            self.headers["PRIVATE-TOKEN"] = self.private_token
-            self.headers.pop("JOB-TOKEN", None)
+            self._auth = _backends.PrivateTokenAuth(self.private_token)
 
         if self.oauth_token:
-            self.headers["Authorization"] = f"Bearer {self.oauth_token}"
-            self.headers.pop("PRIVATE-TOKEN", None)
-            self.headers.pop("JOB-TOKEN", None)
+            self._auth = _backends.OAuthTokenAuth(self.oauth_token)
 
         if self.job_token:
-            self.headers.pop("Authorization", None)
-            self.headers.pop("PRIVATE-TOKEN", None)
-            self.headers["JOB-TOKEN"] = self.job_token
+            self._auth = _backends.JobTokenAuth(self.job_token)
 
-        if self.http_username:
-            self._http_auth = requests.auth.HTTPBasicAuth(
+        if self.http_username and self.http_password:
+            self._auth = requests.auth.HTTPBasicAuth(
                 self.http_username, self.http_password
             )
 
-    @staticmethod
-    def enable_debug() -> None:
+    def enable_debug(self, mask_credentials: bool = True) -> None:
         import logging
-        from http.client import HTTPConnection  # noqa
+        from http import client
 
-        HTTPConnection.debuglevel = 1
+        client.HTTPConnection.debuglevel = 1
         logging.basicConfig()
-        logging.getLogger().setLevel(logging.DEBUG)
+        logger = logging.getLogger()
+        logger.setLevel(logging.DEBUG)
+
+        httpclient_log = logging.getLogger("http.client")
+        httpclient_log.propagate = True
+        httpclient_log.setLevel(logging.DEBUG)
+
         requests_log = logging.getLogger("requests.packages.urllib3")
         requests_log.setLevel(logging.DEBUG)
         requests_log.propagate = True
 
+        # shadow http.client prints to log()
+        # https://stackoverflow.com/a/16337639
+        def print_as_log(*args: Any) -> None:
+            httpclient_log.log(logging.DEBUG, " ".join(args))
+
+        setattr(client, "print", print_as_log)
+
+        if not mask_credentials:
+            return
+
+        token = self.private_token or self.oauth_token or self.job_token
+        handler = logging.StreamHandler()
+        handler.setFormatter(utils.MaskingFormatter(masked=token))
+        logger.handlers.clear()
+        logger.addHandler(handler)
+
     def _get_session_opts(self) -> Dict[str, Any]:
         return {
             "headers": self.headers.copy(),
-            "auth": self._http_auth,
+            "auth": self._auth,
             "timeout": self.timeout,
             "verify": self.ssl_verify,
         }
@@ -644,7 +644,7 @@ class Gitlab:
         verb: str,
         path: str,
         query_data: Optional[Dict[str, Any]] = None,
-        post_data: Optional[Union[Dict[str, Any], bytes]] = None,
+        post_data: Optional[Union[Dict[str, Any], bytes, BinaryIO]] = None,
         raw: bool = False,
         streamed: bool = False,
         files: Optional[Dict[str, Any]] = None,
@@ -716,10 +716,8 @@ class Gitlab:
             retry_transient_errors = self.retry_transient_errors
 
         # We need to deal with json vs. data when uploading files
-        json, data, content_type = self._backend.prepare_send_data(
-            files, post_data, raw
-        )
-        opts["headers"]["Content-type"] = content_type
+        send_data = self._backend.prepare_send_data(files, post_data, raw)
+        opts["headers"]["Content-type"] = send_data.content_type
 
         cur_retries = 0
         while True:
@@ -727,8 +725,8 @@ class Gitlab:
                 result = self._backend.http_request(
                     method=verb,
                     url=url,
-                    json=json,
-                    data=data,
+                    json=send_data.json,
+                    data=send_data.data,
                     params=params,
                     timeout=timeout,
                     verify=verify,
@@ -830,12 +828,9 @@ class Gitlab:
         result = self.http_request(
             "get", path, query_data=query_data, streamed=streamed, **kwargs
         )
+        content_type = utils.get_content_type(result.headers.get("Content-Type"))
 
-        if (
-            result.headers["Content-Type"] == "application/json"
-            and not streamed
-            and not raw
-        ):
+        if content_type == "application/json" and not streamed and not raw:
             try:
                 json_result = result.json()
                 if TYPE_CHECKING:
@@ -874,7 +869,6 @@ class Gitlab:
         path: str,
         query_data: Optional[Dict[str, Any]] = None,
         *,
-        as_list: Optional[bool] = None,  # Deprecated in favor of `iterator`
         iterator: Optional[bool] = None,
         **kwargs: Any,
     ) -> Union["GitlabList", List[Dict[str, Any]]]:
@@ -901,23 +895,6 @@ class Gitlab:
         """
         query_data = query_data or {}
 
-        # Don't allow both `as_list` and `iterator` to be set.
-        if as_list is not None and iterator is not None:
-            raise ValueError(
-                "Only one of `as_list` or `iterator` can be used. "
-                "Use `iterator` instead of `as_list`. `as_list` is deprecated."
-            )
-
-        if as_list is not None:
-            iterator = not as_list
-            utils.warn(
-                message=(
-                    f"`as_list={as_list}` is deprecated and will be removed in a "
-                    f"future version. Use `iterator={iterator}` instead."
-                ),
-                category=DeprecationWarning,
-            )
-
         # Provide a `get_all`` param to avoid clashes with `all` API attributes.
         get_all = kwargs.pop("get_all", None)
 
@@ -931,8 +908,6 @@ class Gitlab:
 
         if iterator and page is not None:
             arg_used_message = f"iterator={iterator}"
-            if as_list is not None:
-                arg_used_message = f"as_list={as_list}"
             utils.warn(
                 message=(
                     f"`{arg_used_message}` and `page={page}` were both specified. "
@@ -1032,8 +1007,10 @@ class Gitlab:
             raw=raw,
             **kwargs,
         )
+        content_type = utils.get_content_type(result.headers.get("Content-Type"))
+
         try:
-            if result.headers.get("Content-Type", None) == "application/json":
+            if content_type == "application/json":
                 json_result = result.json()
                 if TYPE_CHECKING:
                     assert isinstance(json_result, dict)
@@ -1048,7 +1025,7 @@ class Gitlab:
         self,
         path: str,
         query_data: Optional[Dict[str, Any]] = None,
-        post_data: Optional[Union[Dict[str, Any], bytes]] = None,
+        post_data: Optional[Union[Dict[str, Any], bytes, BinaryIO]] = None,
         raw: bool = False,
         files: Optional[Dict[str, Any]] = None,
         **kwargs: Any,
@@ -1094,6 +1071,56 @@ class Gitlab:
                 error_message="Failed to parse the server message"
             ) from e
 
+    def http_patch(
+        self,
+        path: str,
+        *,
+        query_data: Optional[Dict[str, Any]] = None,
+        post_data: Optional[Union[Dict[str, Any], bytes]] = None,
+        raw: bool = False,
+        **kwargs: Any,
+    ) -> Union[Dict[str, Any], requests.Response]:
+        """Make a PATCH request to the Gitlab server.
+
+        Args:
+            path: Path or full URL to query ('/projects' or
+                        'http://whatever/v4/api/projecs')
+            query_data: Data to send as query parameters
+            post_data: Data to send in the body (will be converted to
+                              json by default)
+            raw: If True, do not convert post_data to json
+            **kwargs: Extra options to send to the server (e.g. sudo)
+
+        Returns:
+            The parsed json returned by the server.
+
+        Raises:
+            GitlabHttpError: When the return code is not 2xx
+            GitlabParsingError: If the json data could not be parsed
+        """
+        query_data = query_data or {}
+        post_data = post_data or {}
+
+        result = self.http_request(
+            "patch",
+            path,
+            query_data=query_data,
+            post_data=post_data,
+            raw=raw,
+            **kwargs,
+        )
+        if result.status_code in gitlab.const.NO_JSON_RESPONSE_CODES:
+            return result
+        try:
+            json_result = result.json()
+            if TYPE_CHECKING:
+                assert isinstance(json_result, dict)
+            return json_result
+        except Exception as e:
+            raise gitlab.exceptions.GitlabParsingError(
+                error_message="Failed to parse the server message"
+            ) from e
+
     def http_delete(self, path: str, **kwargs: Any) -> requests.Response:
         """Make a DELETE request to the Gitlab server.
 
